package dk.skat.akfa.transformation.javacallout;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Vector;

/**
 * <p>Implements AKFA header structure. Input is read from a filestream and written to a filestream</p>
 * <p>
 *	Filnavn Maskinelt a-kasse og fagligt kontingent
 * Individnavn Slutindivid a-kasse/fagforening
 * Filnr. 285
 * Individnr. 9998
 * Individstrrelse:  230 (min/max tegn)
 * Start pos. Lngde bytes samt repr. Dataforkortelse Bemrkninger (fx. konstanter)
 * 1            4 (Numerisk)          INDIVIDNR       K: 9998.
 * 5            4 (Numerisk)          BRUGERNR        Fire cifre til identifikation af bruger (indsender). Tildeles af SKAT. Skal vre det samme som i INDIVID 0001.
 * 9            7 (Numerisk)          INDIVID ANTAL   Antal indberettede individer incl. start- og slutindivid.
 * 16         215 (Alfanumerisk)      FILLER  -
 * </p>
 * @author Ren Hjortskov Nielsen, SKAT
 * @version 0.1
 */
public class AKFAXMLMessageFooter extends AKFAXMLMessageBase implements AKFAXMLMessageInterface
{
	private String brugernummer = "";
  private long   individantal = 0L;
	private Vector message = new Vector();

	public AKFAXMLMessageFooter(boolean debug) {
		this.debug = debug;
	}
  
	/**
	 * Produce from 999820000200002--------10--------20--------30--------40--------50--------60--------70--------80--------90-------100-------110-------120-------130-------140-------150-------160-------170-------180-------190-------200-------210--215<p>
	 * <pre></pre>
   * </p>
	 */
  public void readFromFile(FileInputStream fis, FileOutputStream fos)
		throws Exception
	{
		if (debug) System.out.println("+AKFAXMLMessageFooter");
		// Reset message
		if (message!=null) { message.clear(); } else { message = new Vector(); }

    /*
     * 1            4 (Numerisk)          INDIVIDNR       K: 9998.
     * 5            4 (Numerisk)          BRUGERNR        Fire cifre til identifikation af bruger (indsender). Tildeles af SKAT. Skal vre det samme som i INDIVID 0001.
     * 9            7 (Numerisk)          INDIVID ANTAL   Antal indberettede individer incl. start- og slutindivid.
     * 16         215 (Alfanumerisk)      FILLER  -
    */
		readBytes(fis,0,4,0,null);
		brugernummer = new String(readBytes(fis,0,4,0,null),"UTF-8"); // brugernummer
		individantal = Long.parseLong(new String(readBytes(fis,0,7,0,null),"UTF-8")); // individ antal
		readBytes(fis,0,215,0,null);

    // Discard LF
    readLinefeed(fis);

		message.addElement(getBytesFromString("</fase1:IndberetningListe>"));
		message.addElement(getBytesFromString("</fase1:InputParametre>"));
/*
<fase1:AKFAAngivelseFilOpret_I revision="noRevision" xsi:schemaLocation="http://skat.dk/begrebsmodel/2006/09/07/ http://modelhost.ccta.dk/~w09998/generate/TestGenerate/Fase1eService/view/AKFAAngivelseFilOpret_IType.xsd" xmlns:fase1="http://skat.dk/begrebsmodel/2006/09/07/" xmlns:n1="http://www.altova.com/samplexml/other-namespace" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
	<fase1:Kontekst>
		<n1:auto-generated_for_wildcard/>
	</fase1:Kontekst>
	<fase1:MeddelelseIndhold>
		<base64>
		</base64>
	</fase1:MeddelelseIndhold>
</fase1:AKFAAngivelseFilOpret_I>
*/
		if (debug) System.out.println("-AKFAXMLMessageFooter");
  }

  public void writeToFile(FileOutputStream fos)
		throws Exception
	{
		//if (debug) System.out.println("+writeToFile elements="+message.size());
  	for (int i=0; i<message.size();i++) {
  	  byte[] b = new byte[1];
  	  if (message.elementAt(i) instanceof byte[])
			  b = ((byte[])message.elementAt(i));
  	  else if (message.elementAt(i) instanceof Byte)
  	    b[0] = ((Byte)message.elementAt(i)).byteValue();
  	  else
  	    throw new Exception("Unhandled object attempted to be written! "+(message.elementAt(i)));
			//if (debug) System.out.println(new String(b,"UTF-8"));
	    fos.write(b);
		  fos.flush();
		}
		message.clear();
		//if (debug) System.out.println("-writeToFile");
  }

  public String getBrugernummer() {
    return brugernummer;
  }

  public long getIndivider() {
    return individantal;
  }
}