package dk.skat.akfa.transformation.javacallout;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Vector;
import java.nio.channels.FileChannel;

/**
 * <p>Startindvid INDVNR 0001
 * Filnavn  Maskinelt a-kasse og fagligt kontingent
 * Individnavn Startindivid a-kasse/fagforening
 * Filnr. 285
 * Individnr 0001
 * Individstrrelse 230
 * Start pos  Lngde bytes samt repr.  Dataforkortelse Bemrkninger (fx konstanter).
 *  1            4 (Numerisk)           INDIVDNR            K: 0001.
 *  5            4 (Numerisk)           BRUGERNR            Fire cifre til identifikation af bruger (indsender). Tildeles af SKAT.
 *  9           10 (Numerisk)           FILNR               K: 0314317285.
 * 19           27 (Alfanumerisk)       INDSENDERS NAVN     Udfyldes med indsenders navn (det navn, til hvem brugernr. er tildelt).
 * 46           35 (Alfanumerisk)       INDSENDERS ADRESSE  Udfyldes med indsenders adresse.
 * 81          150 (Alfanumerisk)       FILLER
 *
 * Diskette solution
 *  SegmentNummer	             1	4	Individnummer	0001=konstant
 *  AngiverVirksomhedSENummer	 5	8	Foreningens SE-nr	-
 *     	                      13	6	999999	Konstant
 *   	                        19	7	0000000	Konstant
 *  IndsenderNavn	            26	35	Foreningens navn	-
 *  IndsenderGadeAdresse	    61	35	Foreningens adresse	-
 *  IndsenderPostNummer	      96	4	Foreningens postnummer	-
 *  Indsendelser	           133	4	Selvangivelsesr	Formatet: 200X
 *  IndsenderTelefonNummer	 137	8	Telefonnummer	-
 * 
 * </p>
 * @author Ren Hjortskov Nielsen, SKAT
 * @version 0.1
 */
public class AKFAXMLMessageHeader extends AKFAXMLMessageBase implements AKFAXMLMessageInterface
{
  private byte[] angiversenr  = null; // Angiver SEnr.
  private String brugernummer = null;
  private String rowCount     = null;
  private String afgiver      = null;
  private String indsender    = null;

  /**
   * Constructor with variables used for create the start of the AKFA xml
   *
   * @param debug True if logging to system out should be supported
   * @param brugernummer Gotten from the csv file footer
   * @param rowCount Calculated number depitching total row count
   * @param afgiver Provided by the caller
   * @param indsender Provided by the caller
   */
	public AKFAXMLMessageHeader(boolean debug, String brugernummer, int rowCount, String afgiver, String indsender) {
		this.debug = debug;
		this.brugernummer = brugernummer;
		this.rowCount = ""+(rowCount-2); // Do not count start and slut line
		this.afgiver = afgiver;
		this.indsender = indsender;
	}

	/**
	 * Produce from 000120000314317285inds200000 stkn-nr1-------xindsenders gadenavn---------------x--------10--------20--------30--------40--------50--------60--------70--------80--------90-------100-------110-------120-------130-------140-------150<p>
	 * <pre></pre>
   * </p>
	 */
  public void readFromFile(FileInputStream fis, FileOutputStream fos)
		throws Exception
	{
		if (debug) System.out.println("+AKFAXMLMessageHeader");
		// Reset message
		if (message!=null) { message.clear(); } else {
			message = new Vector();
		}

    // Create message template
		message.addElement(getBytesFromString("<fase1:InputParametre xmlns:fase1=\"http://skat.dk/begrebsmodel/2006/09/07/\">"));
		message.addElement(getBytesFromString("<fase1:IndsenderOplysningerInddata>"));
		message.addElement(getBytesFromString("<fase1:Segmentering>"));
		message.addElement(getBytesFromString("<fase1:SegmentNummer>0001</fase1:SegmentNummer>")); //22609998
		// ?? antal poster, krver fuldt gennemlb
		message.addElement(getBytesFromString("<fase1:SegmenterIalt>"));
		message.addElement(getBytesFromString(rowCount));
		message.addElement(getBytesFromString("</fase1:SegmenterIalt>"));
		message.addElement(getBytesFromString("</fase1:Segmentering>"));
    
    payload = readBytes(fis,0,230,0,null);    

    byte[] konstant = getFieldValue(13,13);    
    boolean diskette = (new String(konstant).equals("9999990000000")) ? true : false;
    if (!diskette) {    
      // *  1            4 (Numerisk)           INDIVDNR            K: 0001. Segmentnummer
      // *  5            4 (Numerisk)           BRUGERNR            Fire cifre til identifikation af bruger (indsender). Tildeles af SKAT.
      // *  9           10 (Numerisk)           FILNR               K: 0314317285.
      // * 19           27 (Alfanumerisk)       INDSENDERS NAVN     Udfyldes med indsenders navn (det navn, til hvem brugernr. er tildelt).
      // * 46           35 (Alfanumerisk)       INDSENDERS ADRESSE  Udfyldes med indsenders adresse.
      // * 81          150 (Alfanumerisk)       FILLER
      // Indsender
  		message.addElement(getBytesFromString("<fase1:IndsenderNavn><![CDATA["));
      message.addElement(getBytesFromString("")); // HEADER.INDSENDERS NAVN pos 12
  		message.addElement(getBytesFromString("]]></fase1:IndsenderNavn>"));
  		message.addElement(getBytesFromString("<fase1:IndsenderGadeAdresse><![CDATA["));
      message.addElement(getBytesFromString("")); // HEADER.INDSENDERS ADRESSE pos 9
  		message.addElement(getBytesFromString("]]></fase1:IndsenderGadeAdresse>"));
  		// HEADER.BRUGERNR | FOOTER.BRUGERNR
  		message.addElement(getBytesFromString("<fase1:BrugerNummer>"));
      message.addElement(getBytesFromString(brugernummer));
  		message.addElement(getBytesFromString("</fase1:BrugerNummer>"));
  		message.addElement(getBytesFromString("<fase1:YderligereIndsenderOplysninger>"));
  		message.addElement(getBytesFromString("</fase1:YderligereIndsenderOplysninger>"));
  		message.addElement(getBytesFromString("</fase1:IndsenderOplysningerInddata>"));

      setFieldValue(19,27,9);     // Indsenders navn
      setFieldValue(46,35,12);     // Indsenders adresse

    } else {
      // Diskette solution
      /*
      SegmentNummer	             1	4	Individnummer	0001=konstant
      AngiverVirksomhedSENummer	 5	8	Foreningens SE-nr	-
      	                        13	6	999999	Konstant
      	                        19	7	0000000	Konstant
      IndsenderNavn	            26	35	Foreningens navn	-
      IndsenderGadeAdresse	    61	35	Foreningens adresse	-
      IndsenderPostNummer	      96	4	Foreningens postnummer	-
      Indsendelser	           133	4	Selvangivelsesr	Formatet: 200X
      IndsenderTelefonNummer	 137	8	Telefonnummer	-
      */
      // Indsender
  		message.addElement(getBytesFromString("<fase1:IndsenderNavn><![CDATA["));
      message.addElement(getBytesFromString("")); // HEADER.INDSENDERS NAVN
  		message.addElement(getBytesFromString("]]></fase1:IndsenderNavn>"));
  		message.addElement(getBytesFromString("<fase1:IndsenderGadeAdresse><![CDATA["));
      message.addElement(getBytesFromString("")); // HEADER.INDSENDERS ADRESSE
  		message.addElement(getBytesFromString("]]></fase1:IndsenderGadeAdresse>"));
  		// FOOTER.BRUGERNR
  		message.addElement(getBytesFromString("<fase1:BrugerNummer>"));
      message.addElement(getBytesFromString(brugernummer));
  		message.addElement(getBytesFromString("</fase1:BrugerNummer>"));
  		message.addElement(getBytesFromString("<fase1:YderligereIndsenderOplysninger>"));

  		message.addElement(getBytesFromString("<fase1:IndsenderPostNummer>"));
      message.addElement(getBytesFromString("")); // HEADER.INDSENDERS POSTNUMMER
  		message.addElement(getBytesFromString("</fase1:IndsenderPostNummer>"));

  		message.addElement(getBytesFromString("<fase1:IndsenderTelefonNummer>"));
      message.addElement(getBytesFromString("")); // HEADER.INDSENDERS TLF
  		message.addElement(getBytesFromString("</fase1:IndsenderTelefonNummer>"));
  		message.addElement(getBytesFromString("<fase1:Indsendelser>"));
      message.addElement(getBytesFromString("")); // HEADER.INDSENDERS AAR
  		message.addElement(getBytesFromString("</fase1:Indsendelser>"));
  		message.addElement(getBytesFromString("<fase1:AngiverVirksomhedSENummer>"));
  		message.addElement(getBytesFromString("<fase1:VirksomhedSENummer>"));
      message.addElement(getBytesFromString("")); 
  		message.addElement(getBytesFromString("</fase1:VirksomhedSENummer>"));
  		message.addElement(getBytesFromString("</fase1:AngiverVirksomhedSENummer>"));

  		message.addElement(getBytesFromString("</fase1:YderligereIndsenderOplysninger>"));
  		message.addElement(getBytesFromString("</fase1:IndsenderOplysningerInddata>"));

      setFieldValue(5,8,29);      // Indsenders OPSENR
      setFieldValue(26,27,9);    // Indsenders navn
      setFieldValue(61,35,12);    // Indsenders adresse
      setFieldValue(96,4,19);     // Indsenders postnummer
      setFieldValue(137,8,22);    // Indsenders tlfnr
      setFieldValue(133,4,25);    // r program version
      
    }
    
    // Discard LF
    readLinefeed(fis);

    // Header field populated by body content
		message.addElement(getBytesFromString("<fase1:AfgiverVirksomhedSENummer>"));
		message.addElement(getBytesFromString("<fase1:VirksomhedSENummer>"));
    message.addElement(getBytesFromString(afgiver));
		message.addElement(getBytesFromString("</fase1:VirksomhedSENummer>"));
		message.addElement(getBytesFromString("</fase1:AfgiverVirksomhedSENummer>"));
		message.addElement(getBytesFromString("<fase1:VirksomhedIndsender>"));
		message.addElement(getBytesFromString("<fase1:VirksomhedSENummer>"));
    message.addElement(getBytesFromString(indsender));
		message.addElement(getBytesFromString("</fase1:VirksomhedSENummer>"));
		message.addElement(getBytesFromString("</fase1:VirksomhedIndsender>"));
		message.addElement(getBytesFromString("<fase1:AngivelseInddata>"));
		message.addElement(getBytesFromString("<fase1:AngivelseValgtIndberetningFormKode>2</fase1:AngivelseValgtIndberetningFormKode>")); // <= Provided by NTSE AKFA solution
		message.addElement(getBytesFromString("<fase1:AngivelseAngivelseTid>2001-12-17T09:30:47Z</fase1:AngivelseAngivelseTid>"));
		message.addElement(getBytesFromString("<fase1:AngivelseValuta>DKK</fase1:AngivelseValuta>")); // <= Provided by NTSE AKFA solution
		message.addElement(getBytesFromString("</fase1:AngivelseInddata>"));
		message.addElement(getBytesFromString("<fase1:IndberetningListe>"));

		if (debug) System.out.println("-AKFAXMLMessageHeader");
  }

  /**
   * Method for looking ahead in order to determine the format [diskette | others]
   * 
   * Not used since we exploit fixed length format and thus can load the entire row at once into a byte[] array
   */
  private boolean disketteLookahead(FileInputStream fis)
    throws Exception
  {
    boolean diskette = false;

    FileChannel fc = fis.getChannel();
    long pos = fc.position();
    /*if (!fis.markSupported()) {
      throw new Exception("Lookahead is not possible since mark is not supported!");
    }*/

    fis.mark(30);
    angiversenr = readBytes(fis,0,8,0,null); // Angiver SEnr.
    byte[] k999999     = readBytes(fis,0,6,0,null); // K: 999999
    byte[] k0000000    = readBytes(fis,0,7,0,null); // K: 0000000

    if ((new String(k999999)).equals("999999") && (new String(k0000000)).equals("0000000")) {
      diskette = true;
      if (debug) System.out.println("Using AKFA disketteformat");
    } else {
      //fis.reset();
      fc.position(pos);
    }
    return diskette;
  }

}
